#include "Arduino.h"
#include "encoder.h"
#include "lcd.h"
#include "trx_manager.h"

int Encoder::CLK;
int Encoder::DT;
int Encoder::SW;
int Encoder::PW;
int Encoder::enc_ALast;
int Encoder::enc_AVal;
int32_t Encoder::last_micros_clickdebouncer;
int32_t Encoder::last_micros_rotatedebouncer;

void Encoder::Init(int _clk, int _dt, int _sw, int _pw)
{
  CLK = _clk;
  DT = _dt;
  SW = _sw;
  PW = _pw;
  pinMode (PW, OUTPUT);
  digitalWrite(PW, 1);
  pinMode (DT, INPUT_PULLUP);
  pinMode (CLK, INPUT_PULLUP);
  pinMode (SW, INPUT_PULLUP);
  enc_ALast = digitalRead(CLK);

  attachInterrupt(SW, &Encoder::checkClick, FALLING);
  attachInterrupt(CLK, &Encoder::checkRotate, CHANGE);
}

void Encoder::checkRotate() {
  detachInterrupt(CLK);
  enc_AVal = digitalRead(CLK);
  if (enc_AVal != enc_ALast) { // проверка на изменение значения на выводе А по сравнению с предыдущим запомненным, что означает, что вал повернулся
    enc_ALast = enc_AVal;
    // а чтобы определить направление вращения, нам понадобится вывод В.
    if (digitalRead(DT) != enc_AVal) {  // Если вывод A изменился первым - вращение по часовой стрелке
      EncoderRotated(1);
    } else {// иначе B изменил свое состояние первым - вращение против часовой стрелки
      EncoderRotated(-1);
    }
  }
  attachInterrupt(CLK, &Encoder::checkRotate, CHANGE);
}

void Encoder::checkClick() { //смена разрядности переключения частоты валкодером
  if (digitalRead(SW) == HIGH) return;
  detachInterrupt(SW);

  if ((int32_t)(micros() - last_micros_clickdebouncer) >= 500000) { //защита от дребезга контактов
    if (digitalRead(SW) == LOW)
    {
      if (LCD::mainMenuOpened)
      {
        LCD::menu_main_index = LCD::menu_main_index + 1;
        if (LCD::menu_main_index > MENU_MAIN_COUNT) LCD::menu_main_index = 1;
        if (LCD::menu_main_index < 1) LCD::menu_main_index = 1;
        LCD::displayMainMenu();
      }
      if (!LCD::mainMenuOpened)
      {
        LCD::menu_freq_index = LCD::menu_freq_index + 1;
        if (LCD::menu_freq_index > MENU_FREQ_COUNT) LCD::menu_freq_index = 1;
        if (LCD::menu_freq_index < 1) LCD::menu_freq_index = 1;
        LCD::last_showed_freq = 0;
      }
    }
    last_micros_clickdebouncer = micros();
  }
  attachInterrupt(SW, &Encoder::checkClick, FALLING);
}

void Encoder::EncoderRotated(int direction) //энкодер повернули, здесь обработчик, direction -1 - влево, 1 - вправо
{
  int mode_new = 0;

  if (!LCD::mainMenuOpened)
  {
    switch (LCD::menu_freq_index) {
      case MENU_FREQ_HZ:
        TRX_Manager::setFrequency(TRX_Manager::getFrequency() + 50 * direction);
        break;
      case MENU_FREQ_KHZ:
        TRX_Manager::setFrequency(TRX_Manager::getFrequency() + 1000 * direction);
        break;
      case MENU_FREQ_MHZ:
        TRX_Manager::setFrequency(TRX_Manager::getFrequency() + 1000000 * direction);
        break;
      default:
        break;
    }
  }
  if (LCD::mainMenuOpened)
  {
    switch (LCD::menu_main_index) {
      case MENU_MAIN_GAIN:
        if (direction > 0 || TRX_Manager::gain_level > 0) TRX_Manager::gain_level = TRX_Manager::gain_level + direction;
        if (TRX_Manager::gain_level < 0) TRX_Manager::gain_level = 0;
        if (TRX_Manager::gain_level > 15) TRX_Manager::gain_level = 15;
        LCD::needRedrawMainMenu=true;
        break;
      case MENU_MAIN_AGCSPEED:
        if (direction > 0 || TRX_Manager::agc_speed > 0) TRX_Manager::agc_speed = TRX_Manager::agc_speed + direction;
        if (TRX_Manager::agc_speed < 0) TRX_Manager::agc_speed = 0;
        if (TRX_Manager::agc_speed > 15) TRX_Manager::agc_speed = 15;
        LCD::needRedrawMainMenu=true;
        break;
      default:
        break;
    }
  }
}
