#include "Arduino.h"
#include "fpga.h"
#include "trx_manager.h"
#include "lcd.h"

uint32_t FPGA::fpgadata_tmp;
byte FPGA::fpgadata_stage;

void FPGA::Init()
{
  //шина данных ARDUIBNO-FPGA
  pinMode (FPGA_DATA_CLK, OUTPUT); //clock
  pinMode (FPGA_DATA_SYNC, INPUT_PULLUP); //sync
  setPortsIN();
  fpgadata_stage = 0;
}

void FPGA::fpgadata_clock()
{
  digitalWrite(FPGA_DATA_CLK, 1);
  delay(10);
  if (digitalReadDirect(FPGA_DATA_SYNC))
  {
    fpgadata_stage = 1;
    fpgadata_tmp = 0;
  }
  else
    fpgadata_stage++;

  //приём
  if (fpgadata_stage == 1)
  {
    fpgadata_tmp = readPacket();
    TRX_Manager::s_meter = fpgadata_tmp;
  }
  //передача
  if (fpgadata_stage == 2) //такт для переключения направления передачи
  {
    setPortsOUT();
  }
  if (fpgadata_stage == 2) //идём на такт впереди, чтобы успевать менять значения
  {
    uint8_t tmp_packet = ((TRX_Manager::getFrequencyPhrase() & (B11111111 << 16)) >> 16);
    if (TRX_Manager::getMode() == 1) bitWrite(tmp_packet, 7, 1); else bitWrite(tmp_packet, 7, 0);
    if (TRX_Manager::getMode() == 2) bitWrite(tmp_packet, 6, 1); else bitWrite(tmp_packet, 6, 0);
    writePacket(tmp_packet);
  }
  if (fpgadata_stage == 3)
  {
    writePacket(((TRX_Manager::getFrequencyPhrase() & (B11111111 << 8)) >> 8));
  }
  if (fpgadata_stage == 4)
  {
    writePacket(TRX_Manager::getFrequencyPhrase() & (B11111111));
  }
  if (fpgadata_stage == 5)
  {
    uint8_t tmp_packet = 0;
    bitWrite(tmp_packet, 7, TRX_Manager::preamp);
    if (TRX_Manager::loopback)
      bitWrite(tmp_packet, 6, 0);
    else
      bitWrite(tmp_packet, 6, TRX_Manager::agc);
    bitWrite(tmp_packet, 5, TRX_Manager::loopback);
    bitWrite(tmp_packet, 4, bitRead(TRX_Manager::gain_level, 3));
    bitWrite(tmp_packet, 3, bitRead(TRX_Manager::gain_level, 2));
    bitWrite(tmp_packet, 2, bitRead(TRX_Manager::gain_level, 1));
    bitWrite(tmp_packet, 1, bitRead(TRX_Manager::gain_level, 0));
    writePacket(tmp_packet);
  }
  if (fpgadata_stage == 6)
  {
    uint8_t tmp_packet = 0;
    bitWrite(tmp_packet, 7, bitRead(15 - TRX_Manager::agc_speed, 3));
    bitWrite(tmp_packet, 6, bitRead(15 - TRX_Manager::agc_speed, 2));
    bitWrite(tmp_packet, 5, bitRead(15 - TRX_Manager::agc_speed, 1));
    bitWrite(tmp_packet, 4, bitRead(15 - TRX_Manager::agc_speed, 0));
    bitWrite(tmp_packet, 3, TRX_Manager::ptt);
    writePacket(tmp_packet);
  }
  if (fpgadata_stage == 7) //такт для переключения направления передачи
  {
    setPortsIN();
  }
  digitalWrite(FPGA_DATA_CLK, 0);
  if (fpgadata_stage != 7) fpgadata_clock(); //продолжаем передачу до конца не дожидаясь таймера
}

void FPGA::debug()
{
  Serial.println(fpgadata_tmp, BIN);
}

void FPGA::setPortsIN()
{
  pinMode (FPGA_DATA_D0, INPUT_PULLUP); //data
  pinMode (FPGA_DATA_D1, INPUT_PULLUP); //data
  pinMode (FPGA_DATA_D2, INPUT_PULLUP); //data
  pinMode (FPGA_DATA_D3, INPUT_PULLUP); //data
  pinMode (FPGA_DATA_D4, INPUT_PULLUP); //data
  pinMode (FPGA_DATA_D5, INPUT_PULLUP); //data
  pinMode (FPGA_DATA_D6, INPUT_PULLUP); //data
  pinMode (FPGA_DATA_D7, INPUT_PULLUP); //data
}

void FPGA::setPortsOUT()
{
  pinMode (FPGA_DATA_D0, OUTPUT); //data
  pinMode (FPGA_DATA_D1, OUTPUT); //data
  pinMode (FPGA_DATA_D2, OUTPUT); //data
  pinMode (FPGA_DATA_D3, OUTPUT); //data
  pinMode (FPGA_DATA_D4, OUTPUT); //data
  pinMode (FPGA_DATA_D5, OUTPUT); //data
  pinMode (FPGA_DATA_D6, OUTPUT); //data
  pinMode (FPGA_DATA_D7, OUTPUT); //data
}

uint32_t FPGA::readPacket()
{
  uint32_t fpga_packet = 0;
  fpga_packet = digitalRead(FPGA_DATA_D0);
  fpga_packet = (fpga_packet << 1) | digitalReadDirect(FPGA_DATA_D1);
  fpga_packet = (fpga_packet << 1) | digitalReadDirect(FPGA_DATA_D2);
  fpga_packet = (fpga_packet << 1) | digitalReadDirect(FPGA_DATA_D3);
  fpga_packet = (fpga_packet << 1) | digitalReadDirect(FPGA_DATA_D4);
  fpga_packet = (fpga_packet << 1) | digitalReadDirect(FPGA_DATA_D5);
  fpga_packet = (fpga_packet << 1) | digitalReadDirect(FPGA_DATA_D6);
  fpga_packet = (fpga_packet << 1) | digitalReadDirect(FPGA_DATA_D7);
  return fpga_packet;
}

void FPGA::writePacket(uint8_t packet)
{
  digitalWriteDirect(FPGA_DATA_D7, bitRead(packet, 0));
  digitalWriteDirect(FPGA_DATA_D6, bitRead(packet, 1));
  digitalWriteDirect(FPGA_DATA_D5, bitRead(packet, 2));
  digitalWriteDirect(FPGA_DATA_D4, bitRead(packet, 3));
  digitalWriteDirect(FPGA_DATA_D3, bitRead(packet, 4));
  digitalWriteDirect(FPGA_DATA_D2, bitRead(packet, 5));
  digitalWriteDirect(FPGA_DATA_D1, bitRead(packet, 6));
  digitalWriteDirect(FPGA_DATA_D0, bitRead(packet, 7));
}

inline void FPGA::digitalWriteDirect(int pin, boolean val) {
  if (val) g_APinDescription[pin].pPort -> PIO_SODR = g_APinDescription[pin].ulPin;
  else    g_APinDescription[pin].pPort -> PIO_CODR = g_APinDescription[pin].ulPin;
}

inline int FPGA::digitalReadDirect(int pin) {
  return !!(g_APinDescription[pin].pPort -> PIO_PDSR & g_APinDescription[pin].ulPin);
}

