#include <SPI.h>
#include "Adafruit_GFX.h"
#include <MCUFRIEND_kbv.h>
#include <TouchScreen.h>
#include "Arduino.h"
#include "lcd.h"
#include "trx_manager.h"
#include "encoder.h"
#include "functions.h"

#define BLACK   0x0000
#define BLUE    0x001F
#define RED     0xF800
#define GREEN   0x07E0
#define CYAN    0x07FF
#define MAGENTA 0xF81F
#define YELLOW  0xFFE0
#define WHITE   0xFFFF
//tft.color565(0, green, 0)

//пины тач-пада
#define YP A1  // must be an analog pin, use "An" notation!
#define XM A2  // must be an analog pin, use "An" notation!
#define YM 7   // can be a digital pin
#define XP 6   // can be a digital pin

MCUFRIEND_kbv LCD::tft(A3, A2, A1, A0, A4);
String LCD::freq_string_hz;
String LCD::freq_string_khz;
String LCD::freq_string_mhz;
bool LCD::bandMenuOpened = false;
bool LCD::mainMenuOpened = false;
uint32_t LCD::last_showed_freq = 0;
byte LCD::menu_freq_index = MENU_FREQ_KHZ;
byte LCD::menu_main_index = 1;
boolean LCD::needRedrawMainMenu = false;
int LCD::last_s_meter = 1;

void LCD::Init()
{
  //MY LCD ID 0x5408
  resetTouchpadPins();
  uint16_t ID = tft.readID();
  tft.begin(ID);
  tft.setRotation(1);
  tft.fillScreen(BLACK);

  redraw();
}

void LCD::displayTopButtons(bool redraw) { //вывод верхних кнопок
  if (mainMenuOpened) return;
  resetTouchpadPins();
  if (redraw) tft.fillRect(0, 0, 319, 65, BLACK);
  int color = CYAN;

  //вывод мод
  if (!bandMenuOpened)
  {
    tft.setTextSize(2);
    if (TRX_Manager::getMode() == 0) color = YELLOW;
    tft.fillRoundRect(5, 5, 70, 60, 10, color);
    tft.setCursor(23, 30);
    tft.setTextColor(BLUE, color);
    tft.print("LSB");

    color = CYAN;
    if (TRX_Manager::getMode() == 1) color = YELLOW;
    tft.fillRoundRect(80, 5, 70, 60, 10, color);
    tft.setCursor(98, 30);
    tft.setTextColor(BLUE, color);
    tft.print("USB");

    color = CYAN;
    if (TRX_Manager::getMode() == 2) color = YELLOW;
    tft.fillRoundRect(155, 5, 70, 60, 10, color);
    tft.setCursor(178, 30);
    tft.setTextColor(BLUE, color);
    tft.print("IQ");

    color = CYAN;
    tft.fillRoundRect(245, 5, 70, 60, 10, color);
    tft.setCursor(257, 30);
    tft.setTextColor(BLUE, color);
    tft.print("BAND");
  }

  //вывод диапазонов
  if (bandMenuOpened)
  {
    int32_t freq_mhz = (int32_t)(TRX_Manager::getFrequency() / 1000000);
    tft.setTextSize(2);
    color = CYAN; if (freq_mhz == 1) color = YELLOW;
    tft.fillRoundRect(5, 5, 58, 60, 3, color); //1.8
    tft.setCursor(16, 30); tft.setTextColor(BLUE, color); tft.print("1.8");
    color = CYAN; if (freq_mhz == 3) color = YELLOW;
    tft.fillRoundRect(68, 5, 58, 60, 3, color); //3.5
    tft.setCursor(79, 30); tft.setTextColor(BLUE, color); tft.print("3.5");
    color = CYAN; if (freq_mhz == 7) color = YELLOW;
    tft.fillRoundRect(131, 5, 58, 60, 3, color); //7
    tft.setCursor(155, 30); tft.setTextColor(BLUE, color); tft.print("7");
    color = CYAN; if (freq_mhz == 10) color = YELLOW;
    tft.fillRoundRect(194, 5, 58, 60, 3, color); //10
    tft.setCursor(210, 30); tft.setTextColor(BLUE, color); tft.print("10");
    color = CYAN;
    tft.fillRoundRect(257, 5, 58, 60, 3, color); //BACK
    tft.setCursor(262, 30); tft.setTextColor(BLUE, color); tft.print("BACK");

    color = CYAN; if (freq_mhz == 14) color = YELLOW;
    tft.fillRoundRect(5, 70, 58, 60, 3, color); //14
    tft.setCursor(21, 91); tft.setTextColor(BLUE, color); tft.print("14");
    color = CYAN; if (freq_mhz == 18) color = YELLOW;
    tft.fillRoundRect(68, 70, 58, 60, 3, color); //18
    tft.setCursor(86, 91); tft.setTextColor(BLUE, color); tft.print("18");
    color = CYAN; if (freq_mhz == 21) color = YELLOW;
    tft.fillRoundRect(131, 70, 58, 60, 3, color); //21
    tft.setCursor(148, 91); tft.setTextColor(BLUE, color); tft.print("21");
    color = CYAN; if (freq_mhz == 24) color = YELLOW;
    tft.fillRoundRect(194, 70, 58, 60, 3, color); //24
    tft.setCursor(211, 91); tft.setTextColor(BLUE, color); tft.print("24");
    color = CYAN; if (freq_mhz == 28) color = YELLOW;
    tft.fillRoundRect(257, 70, 58, 60, 3, color); //28
    tft.setCursor(274, 91); tft.setTextColor(BLUE, color); tft.print("28");
  }
}

void LCD::displayBottomButtons() { //вывод верхних кнопок
  resetTouchpadPins();

  int color = CYAN;
  tft.setTextSize(2);
  if (!mainMenuOpened)
  {
    if (TRX_Manager::preamp) color = YELLOW;
    tft.fillRoundRect(5, 185, 50, 50, 10, color);
    tft.setCursor(13, 203);
    tft.setTextColor(BLUE, color);
    tft.print("PRE");

    color = CYAN;
    if (TRX_Manager::agc) color = YELLOW;
    tft.fillRoundRect(60, 185, 50, 50, 10, color);
    tft.setCursor(68, 203);
    tft.setTextColor(BLUE, color);
    tft.print("AGC");

    color = CYAN;
    if (TRX_Manager::loopback) color = YELLOW;
    tft.fillRoundRect(115, 185, 50, 50, 10, color);
    tft.setCursor(117, 203);
    tft.setTextColor(BLUE, color);
    tft.print("LOOP");
  }
  color = CYAN;
  if (mainMenuOpened) color = YELLOW;
  tft.fillRoundRect(264, 185, 50, 50, 10, color);
  tft.setCursor(266, 203);
  tft.setTextColor(BLUE, color);
  tft.print("MENU");
}

void LCD::displayFreqInfo() { //вывод частоты на экран
  if (mainMenuOpened) return;
  if (bandMenuOpened) return;
  if (last_showed_freq == TRX_Manager::getFrequency()) return;
  last_showed_freq = TRX_Manager::getFrequency();
  resetTouchpadPins();
  //закрашиваем лишнее на экране если разрядов не хватает
  if (TRX_Manager::getFrequency() < 10000000)
  {
    tft.fillRect(0, 80, 30, 40, BLACK);
    tft.fillRect(300, 80, 20, 40, BLACK);
  }

  tft.setCursor(30, 80);
  if (TRX_Manager::getFrequency() > 10000000) tft.setCursor(10, 80);
  tft.setTextColor(WHITE, BLACK);
  tft.setTextSize(5);

  //добавляем пробелов для вывода частоты
  freq_string_hz = String(TRX_Manager::getFrequency() % 1000);
  freq_string_khz = String((int32_t)(TRX_Manager::getFrequency() / 1000) % 1000);
  freq_string_mhz = String((int32_t)(TRX_Manager::getFrequency() / 1000000) % 1000000);

  if (menu_freq_index == MENU_FREQ_MHZ) tft.setTextColor(BLACK, WHITE);
  tft.print(freq_string_mhz);
  tft.setTextColor(WHITE, BLACK);
  tft.print(".");

  if (menu_freq_index == MENU_FREQ_KHZ) tft.setTextColor(BLACK, WHITE);
  tft.print(Functions::addSymbols(freq_string_khz, 3, "0", false));
  tft.setTextColor(WHITE, BLACK);
  tft.print(".");

  if (menu_freq_index == MENU_FREQ_HZ) tft.setTextColor(BLACK, WHITE);
  tft.print(Functions::addSymbols(freq_string_hz, 3, "0", false));
}

void LCD::displayStatusInfo() {
  if (mainMenuOpened) return;
  tft.setCursor(10, 130);
  tft.setTextSize(2);
  if (TRX_Manager::ptt)
  {
    tft.setTextColor(RED, BLACK);
    tft.print("TX");
  }
  else
  {
    tft.setTextColor(GREEN, BLACK);
    tft.print("RX");
  }

  int width = 275;
  tft.drawRect(40, 130, width, 15, RED);
  int s_width = (TRX_Manager::s_meter * (width - 2) / 255);
  if (last_s_meter > s_width) s_width = s_width + ((last_s_meter - s_width) / 2); //сглаживаем движение с-метра, добавляя по половине
  if (last_s_meter < s_width) s_width = s_width - ((s_width - last_s_meter) / 2);
  if (last_s_meter > s_width) tft.fillRect(41 + last_s_meter, 131, s_width - last_s_meter, 13, BLACK);
  last_s_meter = s_width;
  tft.fillRect(41, 131, s_width, 13, WHITE);
  tft.setTextSize(1);

  int step = width / 8;
  tft.setTextColor(RED, BLACK);
  tft.setCursor(50 + step * 0, 150);
  tft.print("1");
  tft.setCursor(50 + step * 1, 150);
  tft.print("3");
  tft.setCursor(50 + step * 2, 150);
  tft.print("5");
  tft.setCursor(50 + step * 3, 150);
  tft.print("7");
  tft.setCursor(50 + step * 4, 150);
  tft.print("9");
  tft.setCursor(50 + step * 5, 150);
  tft.print("+20");
  tft.setCursor(50 + step * 6, 150);
  tft.print("+40");
  tft.setCursor(50 + step * 7, 150);
  tft.print("+60");
}

void LCD::displayMainMenu() {
  if (!mainMenuOpened) return;
  displayBottomButtons();

  tft.setCursor(5, 5);
  tft.setTextColor(WHITE, BLACK);
  tft.setTextSize(2);
  tft.print("GAIN:");
  tft.setCursor(200, 5);
  if (menu_main_index == MENU_MAIN_GAIN) tft.setTextColor(BLACK, WHITE);
  tft.setTextSize(2);
  tft.print(Functions::addSymbols(String(TRX_Manager::gain_level), 2, " ", true));

  tft.setCursor(5, 25);
  tft.setTextColor(WHITE, BLACK);
  tft.setTextSize(2);
  tft.print("AGC Speed:");
  tft.setCursor(200, 25);
  if (menu_main_index == MENU_MAIN_AGCSPEED) tft.setTextColor(BLACK, WHITE);
  tft.setTextSize(2);
  tft.print(Functions::addSymbols(String(TRX_Manager::agc_speed), 2, " ", true));
}

void LCD::redraw() {
  resetTouchpadPins();
  tft.fillRect(0, 0, 319, 239, BLACK);
  displayTopButtons(false);
  last_showed_freq = 0;
  displayFreqInfo();
  displayStatusInfo();
  displayBottomButtons();
  displayMainMenu();
}

void LCD::resetTouchpadPins()
{
  //освобождаем пины для экрана
  pinMode(XM, OUTPUT);
  digitalWrite(XM, LOW);
  pinMode(YP, OUTPUT);
  digitalWrite(YP, HIGH);
  pinMode(YM, OUTPUT);
  digitalWrite(YM, LOW);
  pinMode(XP, OUTPUT);
  digitalWrite(XP, HIGH);
}

void LCD::doEvents()
{
  displayFreqInfo();
  displayStatusInfo();
  checkTouchPad();
  if (needRedrawMainMenu) {
    needRedrawMainMenu = false;
    displayMainMenu();
  }
}

void LCD::checkTouchPad()
{
  TouchScreen ts = TouchScreen(XP, YP, XM, YM, 530);
  delay(10);
  TSPoint p = ts.getPoint();
  int x = 0;
  int y = 0;

  if (p.z < 1000 && p.z > 10 && p.x < 990) { //ловим нажатие на экран
    x = map(p.x, 110, 880, 0, 319);
    y = map(p.y, 850, 285, 0, 239);
    Serial.print("X = "); Serial.print(p.x);
    Serial.print("\tY = "); Serial.print(p.y);
    Serial.print("\tPressure = "); Serial.print(p.z);
    Serial.print("\tXX = "); Serial.print(x);
    Serial.print("\tYY = "); Serial.print(y);
    Serial.println("");
  }
  while (x > 0 && y > 0 && p.z != 0) //ждём пока отпустят
  {
    delay(100);
    p = ts.getPoint();
  }

  if (!bandMenuOpened && !mainMenuOpened)
  {
    if (x >= 5 && x <= 75 && y >= 5 && y <= 65) TRX_Manager::setMode(0); //кнопка LSB
    if (x >= 80 && x <= 150 && y >= 5 && y <= 65) TRX_Manager::setMode(1); //кнопка USB
    if (x >= 155 && x <= 225 && y >= 5 && y <= 65) TRX_Manager::setMode(2); //кнопка IQ
    if (x >= 245 && x <= 315 && y >= 5 && y <= 65) {
      bandMenuOpened = true;  //кнопка BAND
      displayTopButtons(true);
    }
  }
  else if (bandMenuOpened && !mainMenuOpened)
  {
    if (x >= 1 && x <= 63 && y >= 1 && y <= 65) {
      TRX_Manager::setFrequency(1850000);  //кнопка 1.8
      bandMenuOpened = false;
      redraw();
    }
    if (x >= 68 && x <= 126 && y >= 1 && y <= 65) {
      TRX_Manager::setFrequency(3570000);  //кнопка 3.5
      bandMenuOpened = false;
      redraw();
    }
    if (x >= 131 && x <= 189 && y >= 1 && y <= 65) {
      TRX_Manager::setFrequency(7100000);  //кнопка 7
      bandMenuOpened = false;
      redraw();
    }
    if (x >= 194 && x <= 252 && y >= 1 && y <= 65) {
      TRX_Manager::setFrequency(10130000);  //кнопка 10
      bandMenuOpened = false;
      redraw();
    }
    if (x >= 257 && x <= 319 && y >= 1 && y <= 65) { //кнопка BACK
      bandMenuOpened = false;
      redraw();
    }
    if (x >= 1 && x <= 63 && y >= 68 && y <= 130) {
      TRX_Manager::setFrequency(14100000);  //кнопка 14
      bandMenuOpened = false;
      redraw();
    }
    if (x >= 68 && x <= 126 && y >= 68 && y <= 130) {
      TRX_Manager::setFrequency(18100000);  //кнопка 18
      bandMenuOpened = false;
      redraw();
    }
    if (x >= 131 && x <= 189 && y >= 68 && y <= 130) {
      TRX_Manager::setFrequency(21100000);  //кнопка 21
      bandMenuOpened = false;
      redraw();
    }
    if (x >= 194 && x <= 252 && y >= 68 && y <= 130) {
      TRX_Manager::setFrequency(24900000);  //кнопка 24
      bandMenuOpened = false;
      redraw();
    }
    if (x >= 257 && x <= 319 && y >= 68 && y <= 130) {
      TRX_Manager::setFrequency(28100000);  //кнопка 28
      bandMenuOpened = false;
      redraw();
    }
  }

  //Нижние кнопки
  if (!mainMenuOpened)
  {
    if (x >= 5 && x <= 55 && y >= 185 && y <= 239) {
      TRX_Manager::preamp = !TRX_Manager::preamp; //кнопка PREAMP
      displayBottomButtons();
    }
    if (x >= 60 && x <= 110 && y >= 185 && y <= 239) {
      TRX_Manager::agc = !TRX_Manager::agc; //кнопка AGC
      displayBottomButtons();
    }
    if (x >= 115 && x <= 165 && y >= 185 && y <= 239) {
      TRX_Manager::loopback = !TRX_Manager::loopback; //кнопка LOOP
      displayBottomButtons();
    }
  }
  if (x >= 264 && x <= 319 && y >= 185 && y <= 239) {
    mainMenuOpened = !mainMenuOpened; //кнопка MENU
    redraw();
  }

  resetTouchpadPins();
}

